<?php
require_once 'RentalAction.php';

interface Debugger {
    public function debug($message);
}

class DebuggerEcho implements Debugger {
    public function debug($message) {
        echo "{$message}\n";
    }
}

class DebuggerLog implements Debugger {
    public function debug($message) {
        error_log("{$message}\n", 3, './RentalCompany.log');
    }
}

class DebuggerVoid implements Debugger {
    public function debug($message) {
        // Alle Meldungen einfach ignorieren
    }
}

class RentalCompany {

    /**
     * Fahrzeuge, die vermietet werden knnen
     *
     * @var array
     */
    protected $fleet = array();

    /**
     * Leihvorgnge
     *
     * @var array
     */
    protected $rentalActions = array();

    /**
     * Debugger Objekt
     *
     * @var Debugger
     */
    protected $debugger;

    /**
     * Konstrukor muss jetzt debugger erzeugen
     *
     */
    public function __construct(Debugger $debugger) {
        $this->debugger = $debugger;
    }

    /**
     * Fgt ein neues Fahrzeug der Flotte hinzu
     *
     * @param string  $id
     * @param Vehicle $vehicle
     */
    public function addToFleet($id, Vehicle $vehicle) {
        $this->fleet[$id]  = $vehicle;
        $this->debug("Neues Auto im Fuhrpark: " . $vehicle->getManufacturer());
    }

    /**
     * Leiht ein Fahrzeug aus
     *
     * @param Vehicle $vehicle
     * @param Customer $customer
     * @return RentalAction
     */
    public function rentVehicle(Vehicle $vehicle, Customer $customer) {
        $vehicleId = array_search($vehicle, $this->fleet);
        if ($vehicleId === false) {
        	throw new UnknownVehicleException();
        }
        if (!$this->isVehicleAvailable($vehicle)) {
        	throw new VehicleNotAvailableException();
        }
        $rentalAction = new RentalAction($vehicle, $customer);
        $this->rentalActions[] = $rentalAction;
        $this->debug("Neuer Mietvorgang: " . $customer->getName() . " leiht " . $vehicle->getManufacturer());

        return $rentalAction;
    }

    /**
     * Gibt ein Fahrzeug zurck
     *
     * @param Vehicle $vehicle
     * @return boolean
     */
    public function returnVehicle(Vehicle $vehicle) {
        foreach ($this->rentalActions as $rentalAction) {
        	if ($rentalAction->getVehicle() !== $vehicle) {
        		continue;
        	}
        	if ($rentalAction->isReturned()) {
        		continue;
        	}
        	$rentalAction->markVehicleReturned();
            $this->debug("Rckgabe: " . $rentalAction->getCustomer()->getName() . " gibt " . $vehicle->getManufacturer() . " zurck.");
        	return true;
        }
        return false;
    }

    /**
     * berprft, ob ein Fahrzeug gerade ausgeliehen werden kann
     *
     * @param Vehicle $vehicle
     * @return boolean
     */
    public function isVehicleAvailable(Vehicle $vehicle) {
        foreach ($this->rentalActions as $rentalAction) {
        	if ($rentalAction->getVehicle() !== $vehicle) {
        		continue;
        	}
        	if ($rentalAction->isReturned()) {
        		continue;
        	}
        	return false;
        }
        return true;
    }

    /**
     * Hilfsmethode fr debugging
     *
     * @param string $message
     */
    protected function debug($message) {
        $this->debugger->debug($message);
    }

    public function accept(Visitor $visitor) {
        $visitor->visitRentalCompany($this);
        foreach ($this->rentalActions as $action) {
            $action->accept($visitor);
        }
    }
}
?>